/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.bsvrz.iav.subsegana.subsegana.events.MesswertDispatcher;
import de.bsvrz.iav.subsegana.subsegana.events.MesswertEvent;
import de.bsvrz.iav.subsegana.subsegana.events.MesswertListener;
import de.bsvrz.sys.funclib.bitctrl.daf.BetriebsmeldungDaten;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.modell.MesswertDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StrassenTeilSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Empfängt Messdaten von Messquerschnitten pro Straßenteilsegment und gibt die
 * gemittelten Werte an die Fuzzyfizierung weiter.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Aggregation extends MesswertDispatcher
implements MesswertListener {

	/** Logger. */
	private final Debug log = Debug.getLogger();

	/** Merkt sich die Messquerschnitte die aggregiert werden sollen. */
	private final Set<MessQuerschnittAllgemein> messQuerschnitte;

	/**
	 * Bestimmt zu den Straßenteilsegmenten die dazugehörigen Messquerschnitte.
	 *
	 * @param systemobjekte
	 *            Menge von Systemobjekten.
	 */
	Aggregation(final Collection<SystemObjekt> systemobjekte) {
		Set<StrassenTeilSegment> strassenTeilSegmente;
		List<StrassenTeilSegment> listeSTS;

		LogTools.log(log, SubSegAnaMsg.InfoStarteAggregation);

		// Straßenteilsegmente filtern
		listeSTS = new ArrayList<>();
		for (final SystemObjekt fo : systemobjekte) {
			if (fo instanceof StrassenTeilSegment) {
				listeSTS.add((StrassenTeilSegment) fo);
			}
		}

		// Alle relevanten Messquerschnitte der Straßenteilsegmente merken
		strassenTeilSegmente = new HashSet<>(listeSTS);
		messQuerschnitte = new HashSet<>();
		for (final StrassenTeilSegment sts : strassenTeilSegmente) {
			for (final MessQuerschnittAllgemein mq : sts
					.getMessQuerschnitte()) {
				messQuerschnitte.add(mq);
			}
		}

		LogTools.log(log, SubSegAnaMsg.ConfigAggregationBereit,
				strassenTeilSegmente.size());
	}

	/**
	 * Aggregiert empfangene Messwerte.
	 */
	@Override
	public void messwertAktualisiert(final MesswertEvent event) {
		if (event.getObjekt() instanceof MessQuerschnittAllgemein) {
			MessQuerschnittAllgemein mqEvent;

			mqEvent = (MessQuerschnittAllgemein) event.getObjekt();
			if (messQuerschnitte.contains(mqEvent)) {
				StrassenTeilSegment sts;
				List<MessQuerschnittAllgemein> mengeMQs;
				MesswertDatum aggregiert;
				Map<String, Float> werte;
				Map<String, Float> anzahl;
				OdVerkehrsDatenKurzZeitMq datensatz;

				// Straßenteilsegment des Messquerschnitts bestimmen und alle
				// anderen Messquerschnitte dieses Straßenteilsegments
				sts = mqEvent.getStrassenTeilSegment();
				mengeMQs = sts.getMessQuerschnitte();

				// Messwerte der Messquerschnitte aggregieren
				werte = new HashMap<>();
				anzahl = new HashMap<>();
				for (final MessQuerschnittAllgemein mq : mengeMQs) {
					MesswertDatum datum;

					datum = mq
							.getOnlineDatensatz(OdVerkehrsDatenKurzZeitMq.class)
							.getDatum(OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse
									.getAspekt());
					if (datum == null) {
						// Wenn noch keine Daten des MQ vorliegen, ignorieren
						continue;
					}
					for (final String messwert : datum.getWerte()) {
						final Number wert = datum.getWert(messwert);

						// Ggf. Wert anlegen
						if (!werte.containsKey(messwert)) {
							werte.put(messwert, 0f);
							anzahl.put(messwert, 0f);
						}

						// Werte addieren, Anzahl der Werte um eins erhöhen
						if (wert != null) {
							werte.put(messwert,
									werte.get(messwert) + wert.floatValue());
							anzahl.put(messwert, anzahl.get(messwert) + 1);
						}
					}
				}

				// Mittelwert bilden
				datensatz = (OdVerkehrsDatenKurzZeitMq) event.getDatensatz();
				aggregiert = datensatz.erzeugeDatum();
				for (final String messwert : werte.keySet()) {
					Integer wert;

					if (anzahl.get(messwert) > 0) {
						wert = Math.round(
								werte.get(messwert) / anzahl.get(messwert));
					} else {
						wert = null;
					}
					aggregiert.setWert(messwert, wert);
				}

				// Datensatz senden
				aggregiert.setZeitstempel(event.getDatum().getZeitstempel());
				fireNeuerMesswert(sts, datensatz, aggregiert);
				LogTools.log(log,
						new BetriebsmeldungDaten(sts.getSystemObject()),
						SubSegAnaMsg.FineNeuerAggregierterWert, sts,
						aggregiert);
			}
		}
	}
}
